@@ -6,6 +6,7 @@
  */
 
 #include "OgreTrays.h"
+#include <sstream>
 
 namespace OgreBites
 {
@@ -425,12 +426,8 @@ SelectMenu::SelectMenu(const Ogre::String &name, const Ogre::DisplayString &capt
 }
 
 void SelectMenu::copyItemsFrom(SelectMenu *other){
-    const Ogre::StringVector& items = other->getItems();
-    Ogre::StringVector::const_iterator it, itEnd;
-    itEnd = items.end();
-    for(it=items.begin(); it != itEnd; it++){
-        this->addItem(*it);
-    }
+    for(const auto& i : other->getItems())
+        addItem(i);
 }
 
 void SelectMenu::setCaption(const Ogre::DisplayString &caption)
@@ -538,17 +535,11 @@ void SelectMenu::selectItem(size_t index, bool notifyListener)
 
 bool SelectMenu::containsItem(const Ogre::DisplayString &item)
 {
-    bool res = false;
-    for (unsigned int i = 0; i < mItems.size(); i++)
-    {
-        if (item == mItems[i])
-        {
-            res = true;
-            break;
-        }
-    }
+    auto i = std::find(mItems.begin(), mItems.end(), item);
+    if (i != mItems.end())
+        return true;
 
-    return res;
+    return false;
 }
 
 void SelectMenu::selectItem(const Ogre::DisplayString &item, bool notifyListener)
@@ -791,7 +782,7 @@ Separator::Separator(const Ogre::String &name, Ogre::Real width)
     }
 }
 
-Slider::Slider(const Ogre::String &name, const Ogre::DisplayString &caption, Ogre::Real width, Ogre::Real trackWidth, Ogre::Real valueBoxWidth, Ogre::Real minValue, Ogre::Real maxValue, unsigned int snaps)
+Slider::Slider(const Ogre::String &name, const Ogre::DisplayString &caption, Ogre::Real width, Ogre::Real trackWidth, Ogre::Real valueBoxWidth, float minValue, float maxValue, unsigned int snaps)
     : mDragOffset(0.0f)
     , mValue(0.0f)
     , mMinValue(0.0f)
@@ -836,7 +827,7 @@ Slider::Slider(const Ogre::String &name, const Ogre::DisplayString &caption, Ogr
     setRange(minValue, maxValue, snaps, false);
 }
 
-void Slider::setRange(Ogre::Real minValue, Ogre::Real maxValue, unsigned int snaps, bool notifyListener)
+void Slider::setRange(float minValue, float maxValue, unsigned int snaps, bool notifyListener)
 {
     mMinValue = minValue;
     mMaxValue = maxValue;
@@ -857,11 +848,11 @@ void Slider::setRange(Ogre::Real minValue, Ogre::Real maxValue, unsigned int sna
     }
 }
 
-void Slider::setValue(Ogre::Real value, bool notifyListener)
+void Slider::setValue(float value, bool notifyListener)
 {
     if (mInterval == 0) return;
 
-    mValue = Ogre::Math::Clamp<Ogre::Real>(value, mMinValue, mMaxValue);
+    mValue = Ogre::Math::Clamp(value, mMinValue, mMaxValue);
 
     setValueCaption(Ogre::StringConverter::toString(mValue));
 
@@ -1193,10 +1184,8 @@ TrayManager::~TrayManager()
 
     destroyAllWidgets();
 
-    for (unsigned int i = 0; i < mWidgetDeathRow.size(); i++)   // delete widgets queued for destruction
-    {
-        delete mWidgetDeathRow[i];
-    }
+    for (auto *w : mWidgetDeathRow) // delete widgets queued for destruction
+        delete w;
     mWidgetDeathRow.clear();
 
     om.destroy(mBackdropLayer);
