@@ -69,13 +69,20 @@ namespace Ogre {
         mFace(face),
         mMipLevel(mipLevel)
     {
-        if(mUsage & TU_RENDERTARGET)
+        if(parentTexture->getUsage() & TU_RENDERTARGET)
         {
-            // Create render target for each slice
-            mSliceTRT.reserve(mDepth);
-            for(size_t zoffset=0; zoffset<mDepth; ++zoffset)
+            if (parentTexture->getUsage() & TU_TARGET_ALL_LAYERS)
             {
-                String name = getNameForRenderTexture(parentTexture->getName());
+                depth = 1;
+                if(face > 0) // only one rendertarget for all layers
+                    return;
+            }
+
+            // Create render target for each slice
+            mSliceTRT.reserve(depth);
+            for(size_t zoffset=0; zoffset<depth; ++zoffset)
+            {
+                String name = getNameForRenderTexture(parentTexture->getName(), zoffset + face);
                 RenderTexture *trt = new D3D11RenderTexture(name, this, zoffset, mDevice);
                 mSliceTRT.push_back(trt);
                 Root::getSingleton().getRenderSystem()->attachRenderTarget(*trt);
@@ -459,6 +466,11 @@ namespace Ogre {
         return mFace;
     }
     //-----------------------------------------------------------------------------    
+    UINT D3D11HardwarePixelBuffer::getMipLevel() const
+    {
+        return mMipLevel;
+    }
+    //-----------------------------------------------------------------------------
     void D3D11HardwarePixelBuffer::createStagingBuffer()
     {
         D3D11Texture *tex = static_cast<D3D11Texture*>(mParentTexture);
